<?php
// $Id: cmis_alfresco.cmis.inc,v 1.1 2009/02/21 03:23:43 cfuller12 Exp $

/**
 * Implementation of cmisapi_getRepositoryInfo method 
 *
 * @todo populate $repository_info with all properties
 */
function cmis_alfresco_cmisapi_getRepositoryInfo() {
  module_load_include('utils.inc', 'cmis_alfresco');

  $response = cmis_alfresco_utils_invoke_service('/api/repository');
  $repo_info = cmis_utils_process_CMIS_xml($response, '/app:service/app:workspace/cmis:repositoryInfo');
  $repository = new stdClass();

  if (false != $repo_info) {
    $repository->repositoryId = cmis_utils_CMIS_xml_get_value($repo_info[0], 'cmis:repositoryId');
    $repository->repositoryName = cmis_utils_CMIS_xml_get_value($repo_info[0], 'cmis:repositoryName');
    $repository->repositoryDescription = cmis_utils_CMIS_xml_get_value($repo_info[0], 'cmis:repositoryDescription');
    $repository->vendorName = cmis_utils_CMIS_xml_get_value($repo_info[0], 'cmis:vendorName');
    $repository->productName = cmis_utils_CMIS_xml_get_value($repo_info[0], 'cmis:productName');
    $repository->productVersion = cmis_utils_CMIS_xml_get_value($repo_info[0], 'cmis:productVersion');
    $repository->rootFolderId = (string) cmis_utils_CMIS_xml_get_value($repo_info[0], 'cmis:rootFolderId');
  }

  return $repository;
}

/**
 * Implementation of cmisapi_getChildren method
 * @param $repositoryId
 * @param $folderId
 */
function cmis_alfresco_cmisapi_getChildren($repositoryId, $folderId) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $folderId = cmis_alfresco_objectId($folderId);
  if ($folderId['noderef_url']) {
    $url = '/api/node/' . $folderId['noderef_url'];
  }
  else {
    $url = $folderId['url'];
  }

  $xml = cmis_alfresco_utils_invoke_service($url . '/children');
  if (false != $xml) {
    return cmis_alfresco_getEntries(cmis_utils_process_CMIS_xml($xml, '//D:entry'));
  }

  return false;
}

/**
 * Implementation of cmisapi_getChildren method
 * @param $repositoryId
 * @param $folderId
 */
function cmis_alfresco_cmisapi_getObjectParents($repositoryId, $folderId) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $folderId = cmis_alfresco_objectId($folderId);
  if ($folderId['noderef_url']) {
    $url = '/api/node/' . $folderId['noderef_url'];
  }
  else {
    $url = $folderId['url'];
  }

  $xml = cmis_alfresco_utils_invoke_service($url . '/parent');
  if (false != $xml) {
    return cmis_alfresco_getEntries(cmis_utils_process_CMIS_xml($xml, '//D:entry'));
  }

  return false;
}

/**
 * Implementation of cmisapi_createDocument method 
 * @param $repositoryId
 * @param $objectTypeId
 * @param $properties
 * @param $folderId
 * @param $content
 * @param $versioningState
 */
function cmis_alfresco_cmisapi_createDocument($repositoryId, $objectTypeId = 'document', $properties = array (), $folderId = null, $content = null, $versioningState = null) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $parentFolderId = cmis_alfresco_objectId($folderId);
  if (!is_array($parentFolderId) || !array_key_exists('noderef_url', $parentFolderId)) {
    drupal_set_message('Unable to find destination folder: ' . $folderId, 'error');
    return false;
  }

  $postvars = '<?xml version="1.0" encoding="utf-8"?>' .
  '<entry xmlns="http://www.w3.org/2005/Atom" xmlns:cmis="http://www.cmis.org/2008/05">' .
  '<title>' . $properties['title'] . '</title>' .
  '<summary>' . $properties['summary'] . '</summary>' .
  '<content type="' . $properties['content-type'] . '">' . base64_encode($content) . '</content>' .
  '<cmis:object>' .
  '<cmis:properties>' .
  '<cmis:propertyString cmis:name="ObjectTypeId">' .
  '<cmis:value>' . $objectTypeId . '</cmis:value>' .
  '</cmis:propertyString>' .
  '</cmis:properties>' .
  '</cmis:object>' .
  '</entry>';

  $header[] = 'Content-type: application/atom+xml;type=entry';
  $header[] = 'Content-length: ' . strlen($postvars);
  $header[] = 'MIME-Version: 1.0';

  $xml = cmis_alfresco_utils_invoke_service('/api/node/' . $parentFolderId['noderef_url'] . '/children', 'ticket', $header, 'CUSTOM-POST', $postvars);
  $entry = cmis_utils_process_CMIS_xml($xml, '/D:entry');
  $entry = $entry[0][0];

  $objectId = cmis_alfresco_objectId((string)$entry->id);

  return $objectId['noderef'];
}

/**
 * Implementation of cmisapi_getProperties method
 * @param $repositoryId
 * @param $objectId
 */
function cmis_alfresco_cmisapi_getProperties($repositoryId, $objectId) {
  module_load_include('utils.inc', 'cmis_alfresco');
  $objectId = cmis_alfresco_objectId($objectId);

  if ($xml = cmis_alfresco_utils_invoke_service('/api/node/' . $objectId['noderef_url'])) {
    if ($entries = cmis_alfresco_getEntries(cmis_utils_process_CMIS_xml($xml, '//D:entry'))) {
      return $entries[0];
    }
  }

  watchdog(WATCHDOG_ERROR, 'Unknown objectId ' . $objectId);
  return false;
}

/**
 * Implemetation of cmisapi_getContentStream method
 * @param $repositoryId
 * @param $objectId
 */
function cmis_alfresco_cmisapi_getContentStream($repositoryId, $objectId) {
  module_load_include('utils.inc', 'cmis_alfresco');
  $objectId = cmis_alfresco_objectId($objectId);
  return cmis_alfresco_utils_invoke_service('/api/node/content/' . $objectId['noderef_url']);
}

/**
 * Implementation of cmisapi_setContentStream method
 * @param $repositoryId
 * @param $objectId
 * @param $overwriteFlag
 * @param $content
 * @param $properties
 */
function cmis_alfresco_cmisapi_setContentStream($repositoryId, $objectId, $overwriteFlag = true, $content = null, $properties = array ()) {
  module_load_include('utils.inc', 'cmis_alfresco');
  $postvars = '<?xml version="1.0" encoding="utf-8"?>' .
  '<entry xmlns="http://www.w3.org/2005/Atom" xmlns:cmis="http://www.cmis.org/2008/05">' .
  '<title>' . $properties['title'] . '</title>' .
  '<content type="' . $properties['content-type'] . '">' . $content . '</content>' .
  '</entry>';

  $header[] = 'Content-type: application/atom+xml;type=entry';
  $header[] = 'Content-length: ' . strlen($postvars);
  $header[] = 'MIME-Version: 1.0';

  $objectId = cmis_alfresco_objectId($objectId);
  $xml = cmis_alfresco_utils_invoke_service('/api/node/' . $objectId['noderef_url'], 'ticket', $header, 'CUSTOM-PUT', $postvars);
  return $xml;
}

/**
 * Implementation of cmisapi_query method  
 * @param $repositoryId
 * @param $statement
 * @param $searchAllVersions
 * @param $includeAllAllowableActions
 * @param $includeRelationships
 * @param $maxItems
 * @param $skipCount
 * @return unknown_type
 */
function cmis_alfresco_cmisapi_query($repositoryId, $statement, $searchAllVersions = false, $includeAllAllowableActions = false, $includeRelationships = null, $maxItems = 0, $skipCount = 0) {
  module_load_include('utils.inc', 'cmis_alfresco');

  $postvars = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>' .
  '<query xmlns="http://www.cmis.org/2008/05">' .
  '<statement>' . $statement . '</statement>' .
  '<searchAllVersions>' . $searchAllVersions . '</searchAllVersions>' .
  '<maxItems>' . $maxItems . '</maxItems>' .
  '<skipCount>' . $maxItems . '</skipCount>' .
  '<returnAllowableActions>' . $includeAllAllowableActions . '</returnAllowableActions>' .
  '</query>';

  $header[] = 'Content-type: application/cmisquery+xml';
  $header[] = 'Content-length: ' . strlen($postvars);
  $header[] = 'MIME-Version: 1.0';

  $xml = cmis_alfresco_utils_invoke_service('/api/query', 'ticket', $header, 'CUSTOM-POST', $postvars);
  if (false != $xml) {
    return cmis_alfresco_getEntries(cmis_utils_process_CMIS_xml($xml, '//D:entry'));
  }

  return false;
}